<?

class GenericParser extends BaseObject {

 var $d_showErrors;
 var $d_evaluate;
 var $d_requestor;
 var $p_parser;
 var $p_alias;
 var $d_parseTagResult;
 var $d_parseTagParams;
 var $d_tagPosHash;
 var $d_parserkeyname;
 var $d_errorPrefix;
 var $d_useSimpleTags;
 var $d_parts;

 function GenericParser() {
 $this->d_showErrors = false;
 $this->d_evaluate = true;
 $this->p_alias = array();
 $this->p_parser = array();
 $this->d_parseTagResult = false;
 $this->d_parseTagParams = false;
 $this->d_tagPosHash = array();
 $this->d_useSimpleTags = false;
 $this->d_parts = false;
 }

 function setShowErrors($val) {
 $this->d_showErrors = $val;
 }
 

 function doShowErrors() {
 return $this->d_showErrors;
 }

 function setEvaluate($value) {
 $this->d_evaluate = $value;
 }

 function doEvaluate() {
 return $this->d_evaluate;
 }

 function setParseTagResult($value) {
 $this->d_parseTagResult = $value;
 }

 function doParseTagResult() {
 return $this->d_parseTagResult;
 }

 function setParseTagParams($value) {
 $this->d_parseTagParams = $value;
 }

 function doParseTagParams() {
 return $this->d_parseTagParams;
 }

 function setUseSimpleTags($value) {
 $this->d_useSimpleTags = $value;
 }

 function doUseSimpleTags() {
 return $this->d_useSimpleTags;
 }

 function &getObjectParser($type) {
 $obj = false;
 $key = $this->_getParserKey($type);
 if ($key) $obj =& $this->p_parser[$key];
 return $obj;
 }

 function setObjectParser($type, &$parser) {
 $key = $this->_getParserKey($type);
 if (!$key) $key = $type;
 $this->_initObjectParser($parser);
 $parser->setType($key);
 $this->p_parser[$key] =& $parser;
 }

 function addObjectParser($type, &$parser, $alias = array()) {
 $this->_initObjectParser($parser);
 $parser->setType($type);
 $this->p_parser[$type] =& $parser;
 while (list($key,$val) = each($alias)) {
 $this->p_alias[$val] = $type;
 }
 }

 function unsetObjectParser($type) {
 
 }

 function parse($code) {
 static $parseDepth = 0;

 if ($parseDepth > 10) return $code;
 $parseDepth++;

 $parts = array();
 $endpos = -1;
 $startpos = -1;

 $tagHash = $this->_buildTagPosHash(strtolower($code));
 if (count($tagHash) == 0) {
 $parseDepth --;
 return $code;
 }

 
 $pos = 0;
 $parts = $this->_buildStructure($code, $tagHash, $pos, false);
 
 $rs = $this->_parseStructure($parts);
 
 $parseDepth--;
 
 unset($this->parts);
 $this->parts =& $parts;
 
 return $rs[0]["code"];
 }
 
 function findParserTags($type, $keyname, $firstOnly = false) {
 return $this->_findParserTags($type, $keyname, $this->parts, $firstOnly);
 }
 
 function _findParserTags($type, $keyname, $parts, $firstOnly) {
 $tags = array();
 
 if (!is_array($parts)) return array();
 
 foreach ($parts as $part) {
 if (!is_a($part, "ParserTag")) continue;
 
 if ($part->getTagType() == $type && $part->getKeyname() == $keyname) {
 $tags[] = $part;
 if ($firstOnly) break;
 }
 
 $_tags = $this->_findParserTags($type, $keyname, $part->getParts(), $firstOnly);
 foreach ($_tags as $tag) {
 $tags[] = $tag;
 if ($firstOnly) break;
 }
 }
 return $tags;
 }
 
 function _buildStructure($code, &$tagHash, &$pos, $intag) {

 $isend = false;
 $parts = array();
 $tagparser = new TagParser($this->d_useSimpleTags);
 $tagname = ($intag!==false)?$intag:"";
 while (!is_false($nextPos = $this->_getNextTag($tagHash, $pos, $tagpos, $isend))) {

 
 if ($nextPos > $pos) {
 $pcode = substr($code, $pos, $nextPos - $pos);
 $parts[] = new StringObject($pcode);
 $pos = $nextPos;
 }
 
 
 $tagparser->startTag();
 $tagend = $tagparser->getTagEnd($code, $tagpos, true);
 if ($tagend == -1) {
 $parts[] = new NodeException(i18n("Could not find closing '>' for <b>{0}</b>.", 0, $tagname, substr($code, $pos)));
 $parts[] = new StringObject(substr($code, $pos));
 $pos = strlen($code);
 return $parts;
 }

 
 $pcode = substr($code, $tagpos, $tagend - $tagpos);
 $hasparts = false;
 if ($pcode && !$isend) {
 
 if ($this->d_parseTagParams) {
 $partHash = $this->_buildPartTagPosHash($tagHash, $tagend, $tagpos);
 if (count($partHash) > 0) {
 $_pos = 0;
 $struct = $this->_buildStructure($pcode, $partHash, $_pos, false);
 $rs = $this->_parseStructure($struct);
 $pcode = $rs[0]["code"];
 $tagparser->startTag();
 $tagparser->parseTag($pcode);
 }
 }
 $params = $tagparser->getParts();
 $hasparts = !$tagparser->isClosed();

 
 $tags = $this->_generateTags($params, $isend, $hasparts, $pcode);
 } else {
 $tags = array();
 }
 $pos = $tagend + 1;

 
 if ($hasparts && !$isend) {
 $props = $this->_buildStructure($code, $tagHash, $pos, $pcode);
 $this->_addParts($tags, $props, $hasparts);
 }

 
 reset($tags);
 while (list($key,$tag) = each($tags)) {
 $parts[] = $tag;
 }
 
 
 if (!$intag && $isend) {
 
 $parts[] = new NodeException(i18n("Found end tag without begin tag"));
 } else if ($intag && $isend) {
 
 return $parts;
 }

 }

 if ($intag) {
 
 $parts[] = new NodeException(i18n("Could not find closing tag for <b>{0}</b>.", 0, $tagname));
 } else if ($pos < strlen($code)) {
 
 $pcode = substr($code, $pos);
 $parts[] = new StringObject($pcode);
 $pos = strlen($code);
 }

 return $parts;
 }

 function _parseStructure($parts, $parseKeys = false) {
 $ckey = 0;
 $code = array($ckey=>array("code"=>"","name"=>"","params"=>array()));
 reset($parts);
 while (list($key,$part) = each($parts)) {
 if ($parseKeys != false && !is_a($part, "parsersplitter")) {
 $name = $ckey == 0 ? "main" : $code[$ckey]["name"];
 if (is_array($parseKeys)) {
 if (!in_array($name, $parseKeys)) continue;
 } else if ($parseKeys === true) {
 if ($name != "main") continue;
 } else if ($parseKeys != $name) {
 continue;
 }
 }
 
 
 if (is_a($part, "stringobject")) {

 $code[$ckey]["code"] .= $this->_handleCode($part);

 } else if (is_a($part, "parsertag")) {
 
 $obj =& $this->getObjectParser($part->getTagType());
 $obj->setKeyname($part->getKeyname());

 
 $tagparts = $part->getParts();

 
 $keynames = array("main"=>$part->getProperties());
 foreach ($tagparts as $k=>$p) {
 if (is_a($p, "parsersplitter")) {
 $keynames[$p->getTagType()] = $p->getProperties();
 }
 }
 $tagkeys = $obj->checkCodeParts($keynames);
 
 $tags = $this->_parseStructure($tagparts, $tagkeys);

 $pcode = $tags[0]["code"];
 unset($tags[0]);
 reset($tags);
 $obj->clearCodeParts();
 while (list($k,$v) = each($tags)) {
 $obj->addCodePart($v["name"],$v["code"],$v["params"]);
 }

 if ($obj->keyExists()) {
 
 $obj->setCodeParts($tagkeys);
 $rs = $obj->parse($pcode, $part->getProperties());
 if (!is_array($rs)) $rs = array($rs);
 $_code = $this->_generateCode($rs);

 
 if ($this->d_parseTagResult) {
 $_code = $this->parse($_code);
 }

 $code[$ckey]["code"] .= $_code;
 } else {
 $rs = new NodeException(i18n("Key '{0}' for '{1}' does not exist.",0,$part->getKeyname(), $part->getTagType()) );
 $code[$ckey]["code"] .= $this->_generateCode(array($rs));
 }
 unset($obj);
 } else if (is_a($part, "parsersplitter")) {
 
 $ckey++;
 $code[$ckey] = array("code"=>"","name"=>$part->getTagType(),"params"=>$part->getProperties());

 } else {
 
 $code[$ckey]["code"] .= $this->_generateCode(array($part));
 }

 }

 return $code;
 }

 function _getParserKey($alias) {
 $key = strtolower($alias);
 if (isset($this->p_parser[$key])) return $key;
 if ($val = $this->p_alias[$key]) return $val;
 return false;
 }

 function _addParts(&$tags, $parts, $partkeys) {
 $a =& $tags;
 reset($partkeys);
 while (list($key,$val) = each($partkeys)) {
 $a =& $tags[$val];
 }
 if (is_a($a, "parsertag")) {
 $a->setParts($parts);
 } else {
 return false;
 }
 return true;
 }

 









 function _getNextTag(&$tagHash, $pos, &$tagpos, &$endtag) {
 while (list($_pos, $val) = each($tagHash)) {
 if ($_pos < $pos) continue;
 $endtag = $val["isend"];
 $tagpos = $_pos + $val["taglen"];
 return $_pos;
 }
 return false;
 }

 function _buildTagPosHash($lcode, $pos = 0) {
 $hash = array();
 
 $tags = $this->_getTagPrefixList();
 while (list($tag,$val) = each($tags)) {
 $_pos = $pos;
 $l = strlen($tag);
 while (($p = strpos($lcode, $tag, $_pos)) !== false) {
 $hash[$p] = $val;
 $_pos = $p + $l;
 }
 }
 ksort($hash);
 reset($hash);
 return $hash;
 }

 function _buildPartTagPosHash(&$tagHash, $toPos, $offset) {
 $rs = array();
 while (list($pos,$val) = each($tagHash)) {
 if ($pos < $toPos && $pos >= $offset) {
 $rs[$pos - $offset] = $val;
 } else if ($pos >= $toPos) {
 if (current($tagHash) === false) {
 end($tagHash);
 } else {
 prev($tagHash);
 }
 return $rs;
 }
 }
 end($tagHash);
 return $rs;
 }

 function getParserKey() { }

 function _initObjectParser(&$parser) {
 $parser->setParser($this);
 }

 function _generateCode($tags) { 
 $code = "";
 foreach ($tags as $tag) {
 if (is_a($tag,"emessage")) {
 $code .= $this->_handleMessage($tag);
 } else if (is_a($tag,"baseobject")) {
 $code .= $this->_handleObject($tag);
 }
 }
 return $code;
 }

 function _generateTags($params, &$isend, &$hasparts) { }

 function _getTagPrefixList() { }

 function _handleCode($tag) {
 return $tag->toString();
 }

 function getErrorPrefix()
 {
 return $this->d_errorPrefix; 
 }
 
 function setErrorPrefix( $prefix )
 {
 $this->d_errorPrefix = $prefix;
 }
 
 function _handleMessage($tag) {
 global $ICO_ALERT, $ICO_INFO;
 $code = "";
 if ($this->d_showErrors && GUIFactory::isInitialized()) {
 $message = i18n(get_class($tag)).": ".$this->d_errorPrefix.$tag->getMessage();
 $alert = new AlertStreamInfo();
 $info = new InfoStreamInfo();
 if ( is_a( $tag, "parsermessage" ) )
 $info->addMessage(0, $ICO_INFO, $message, $tag->getDetailMessage());
 else
 $alert->addMessage( 0, $ICO_ALERT, $message, $tag->getDetailMessage());
 }
 return $code;
 }

 function _handleObject($tag) {
 return $tag->toString();
 }

}

?>
